const express = require('express');

const enrollmentApp = express.Router();

const libs = `${process.cwd()}/libs`;

const log = require(`${libs}/log`)(module);

const squel = require('squel');

const query = require(`${libs}/middlewares/query`);

const response = require(`${libs}/middlewares/response`);

const parseParams = require(`${libs}/middlewares/parseParams`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

let rqf = new ReqQueryFields();

// Complete range of the enrollments dataset.
// Returns a tuple of start and ending years of the complete enrollments dataset.
enrollmentApp.get('/year_range', (req, res, next) => {
    req.sql.from('turma')
    .field('MIN(turma.ano_censo)', 'start_year')
    .field('MAX(turma.ano_censo)', 'end_year');

    next();
}, query, response('range'));

// Returns all educational levels avaible
enrollmentApp.get('/education_level', (req, res, next) => {
    req.sql.from('etapa_ensino')
    .field('pk_etapa_ensino_id', 'id')
    .field('desc_etapa', 'name');

    next();
}, query, response('education_level'));

// Returns all adm dependencies
enrollmentApp.get('/adm_dependency', (req, res, next) => {
    req.sql.from('dependencia_adm')
    .field('pk_dependencia_adm_id', 'id')
    .field('nome', 'name');

    next();
}, query, response('adm_dependency'));

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'adm_dependency',
    table: 'dependencia_adm',
    tableField: 'nome',
    resultField: 'adm_dependency_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'pk_dependencia_adm_id'
    },
    join: {
        primary: 'pk_dependencia_adm_id',
        foreign: 'fk_dependencia_adm_id',
        foreignTable: 'turma'
    }
}).addValue({
    name: 'education_level',
    table: 'etapa_ensino',
    tableField: 'desc_etapa',
    resultField: 'education_level',
    where: {
        relation: '=',
        type: 'integer',
        field: 'pk_etapa_ensino_id'
    },
    join: {
        primary: 'pk_etapa_ensino_id',
        foreign: 'fk_etapa_ensino_id',
        foreignTable: 'turma'
    }
}).addValue({
    name: 'region',
    table: 'regiao',
    tableField: 'nome',
    resultField: 'region_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'pk_regiao_id'
    },
    join: {
        primary: 'pk_regiao_id',
        foreign: 'fk_regiao_id',
        foreignTable: 'turma'
    }
}).addValue({
    name: 'state',
    table: 'estado',
    tableField: 'nome',
    resultField: 'state_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'pk_estado_id'
    },
    join: {
        primary: 'pk_estado_id',
        foreign: 'fk_estado_id',
        foreignTable: 'turma'
    }
}).addValue({
    name: 'city',
    table: 'municipio',
    tableField: 'nome',
    resultField: 'city_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'pk_cod_ibge'
    },
    join: {
        primary: 'pk_cod_ibge',
        foreign: 'fk_municipio_id',
        foreignTable: 'turma'
    }
}).addValue({
    name: 'school',
    table: 'escola',
    tableField: 'cod_entidade',
    resultField: 'school_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'cod_entidade'
    },
    join: {
        primary: 'cod_entidade',
        foreign: 'cod_entidade',
        foreignTable: 'turma'
    }
}).addValue({
    name: 'location',
    table: 'localizacao',
    tableField: 'descricao',
    resultField: 'location_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'pk_localizacao_id'
    },
    join: {
        primary: 'pk_localizacao_id',
        foreign: 'fk_localizacao_id',
        foreignTable: 'turma'
    }
}).addValue({
    name: 'city',
    table: 'municipio',
    tableField: 'nome',
    resultField: 'city_name',
    where: {
        relation: '=',
        type: 'integer',
        field: 'pk_cod_ibge'
    },
    join: {
        primary: 'pk_cod_ibge',
        foreign: 'fk_municipio_id',
        foreignTable: 'turma'
    }
}).addValue({
    name: 'min_year',
    table: 'turma',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'max_year',
    table: 'turma',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_censo'
    }
});

enrollmentApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    log.debug(req.sql.toParam());
    req.sql.field('COALESCE(SUM(num_matriculas), 0)', 'total')
    .field("'Brasil'", 'name')
    .field('turma.ano_censo', 'year')
    .from('turma')
    .group('turma.ano_censo')
    .order('turma.ano_censo');
    next();
}, query, response('enrollment'));

module.exports = enrollmentApp;
